// a basic "bar chart" implementation using javascript and "canvas".
// input to "IACanvas.barChart()" function is an array of integers...

var IACanvas = 
{
  ids:0,
  showzeros:true, // can be overridden by invoker!
  centerdiv:false,// can be overridden by invoker!

  graphXYpoints:function(label, wd, ht, xys)
  {
    var minx = xys[0][0];
    var maxx = xys[0][0];
    for (var i=0, pt; pt=xys[i]; i++)
    {
      minx = Math.min(minx, pt[0]);
      maxx = Math.max(maxx, pt[0]);
    }

    var vals = Array();
    var len = maxx - minx + 1;
    for (var i=0; i<len; i++)
      vals[i]=0;
    
    for (var i=0, pt; pt=xys[i]; i++)
      vals[pt[0]-minx] = pt[1];

    this.log(vals);
    this.log(minx, maxx);
    this.insertBarChart(label, wd, ht, vals, []);
  },
  
  insertBarChart:function(label, wd, ht, vals, colors, id)
  {
    //this.log(vals);
    if (typeof(id)=='undefined')
      var appendee = document.getElementsByTagName('body')[0];
    else
      var appendee = document.getElementById(id);

    var obj=document.createElement('div');
    obj.style.border = '1px solid gray';
    obj.style.padding = '10px';
    obj.style.width = wd + 50;
    if (this.centerdiv)
    {
      obj.style.marginLeft  = 'auto';
      obj.style.marginRight = 'auto';
      obj.style.marginTop   = '10px';
      obj.style.marginBottom= '10px';
    }
    else
    {
      obj.style.margin = '10px';
      obj.style.cssFloat = 'left';
    }
    obj.innerHTML = '<center><h3>'+label+'</h3></center>';
    appendee.appendChild(obj);
    
    var can = document.createElement('canvas');
    var cid = 'canvas' + this.ids.toString();
    this.ids++;
    can.setAttribute('id', cid);
    can.setAttribute('width', wd);
    can.setAttribute('height', ht);
    can.style.width = wd;
    can.style.height = ht;
    obj.appendChild(can);


    IACanvas.barChart(cid, vals, colors);
  },
  
  
  barChart:function(id, vals,
                    colors) //optional -- 1-to-1 corresp to vals; else [] xxx
  {
    //this.log(vals);
    var maxval = null;
    var minval = null;
    for (var i in vals)
    {
      var val=parseFloat(vals[i]);
      if (maxval==null  ||  Math.ceil(val)  > maxval)
        maxval = Math.ceil(val);
      if (minval==null  ||  Math.floor(val) < minval)
        minval = Math.floor(val);
    }
    
    
    var obj = document.getElementById(id);
    var ctx = obj.getContext('2d');

    // canvas obj dimens
    var wd = parseInt(obj.style.width);
    var ht = parseInt(obj.style.height);


    var wd_bar = Math.floor(wd / vals.length);

    // number of x points, then min max of y points
    /*
    this.log('# vals: '+vals.length);
    this.log('minval: '+minval);//xxx do something with -- assuming 0 now
    this.log('maxval: '+maxval);
    this.log('bar width: '+wd_bar);
    */
    

    //ctx.fillRect(0,0,wd,ht);   //  draw a black rectangle with default settings

    // bar color:
    ctx.fillStyle = '#09F';

    var n=0;
    for (var i in vals)
    {
      if (!this.showzeros  &&  vals[i]==0)
      {
        n++;
        continue;
      }
      
      // alt -- for a random color:
      // ctx.fillStyle = '#'+ this.toHex(Math.round(Math.random()*255))+ this.toHex(Math.round(Math.random()*255))+ this.toHex(Math.round(Math.random()*255));

      if (colors.length)
        ctx.fillStyle = colors[i];

      // NOTE: subtract 10 from the total available pixels in height so we can
      // add a 7pt font text number above it
      var y = ht - Math.round((ht-10) * vals[i] / maxval);
      var a=[n*wd_bar,  //x
             y,         //y
             wd_bar-2,  //wd
             Math.max(ht-y-1,2)];   //ht
      //this.log(a);
      ctx.fillRect(a[0],a[1],a[2],a[3]);

      var texty = a[1]-10;
      if (texty>15  &&  wd_bar<=15  &&  n%2==0)
        texty -= 15;
      this.drawString(ctx, vals[i].toString(), "#000", 6, a[0], texty);
      
      n++;
    }
    ctx.save();
  },

  
  
  // from http://www.netzgesta.de/S5/canvas-text.html
  deg2rad:function(degrees)
  {
    return Math.PI *degrees/180;
  },

  // from http://www.netzgesta.de/S5/canvas-text.html
  drawString:function(ctx, txt, col, fh, tx, ty)
  {
    var fw = fh*0.666666; var lw = fh*0.125;  
    var ls = lw/2; var xp = 0; var cr = lw; 
    ctx.lineCap = "round"; ctx.lineJoin = "round"
    ctx.lineWidth = lw; ctx.strokeStyle = col;
    for (var i = 0; i < txt.length; i++) {
      this.drawSymbol(ctx, txt[i], ls, tx+xp, ty, fw, fh);
      xp += (txt[i]!="."?fw+cr:(fw/2)+cr);
    }
  },

  // from http://www.netzgesta.de/S5/canvas-text.html
  drawSymbol:function(ctx, symbol, fc, cx, cy, cw, ch)
  {
    ctx.beginPath();
    switch (symbol) {
    case "0":
      ctx.moveTo(cx+fc,cy+(ch*0.333333));
      ctx.arc(cx+(cw/2),cy+(cw/2),(cw/2)-fc,this.deg2rad(180),0, false);
      ctx.arc(cx+(cw/2),(cy+ch)-(cw/2),(cw/2)-fc,0,this.deg2rad(180), false);
      ctx.closePath();
      break;
    case "1":
      ctx.moveTo(cx+(cw*0.1)+fc,cy+ch-fc);
      ctx.lineTo(cx+cw-fc,cy+ch-fc);
      ctx.moveTo(cx+(cw*0.666666),cy+ch-fc);
      ctx.lineTo(cx+(cw*0.666666),cy+fc);
      ctx.lineTo(cx+(cw*0.25),cy+(ch*0.25));
      break;
    case "2":
      ctx.moveTo(cx+cw-fc,cy+(ch*0.8));
      ctx.lineTo(cx+cw-fc,cy+ch-fc);
      ctx.lineTo(cx+fc,cy+ch-fc);
      ctx.arc(cx+(cw/2),cy+(cw*0.425),(cw*0.425)-fc,this.deg2rad(45),this.deg2rad(-180), true);
      break;
    case "3":
      ctx.moveTo(cx+(cw*0.1)+fc,cy+fc);
      ctx.lineTo(cx+(cw*0.9)-fc,cy+fc);
      ctx.arc(cx+(cw/2),cy+ch-(cw*0.5),(cw*0.5)-fc,this.deg2rad(-90),this.deg2rad(180), false);
      break;
    case "4":
      ctx.moveTo(cx+(cw*0.75),cy+ch-fc);
      ctx.lineTo(cx+(cw*0.75),cy+fc);
      ctx.moveTo(cx+cw-fc,cy+(ch*0.666666));
      ctx.lineTo(cx+fc,cy+(ch*0.666666));
      ctx.lineTo(cx+(cw*0.75),cy+fc);
      ctx.moveTo(cx+cw-fc,cy+ch-fc);
      ctx.lineTo(cx+(cw*0.5),cy+ch-fc);
      break;
    case "5":
      ctx.moveTo(cx+(cw*0.9)-fc,cy+fc);
      ctx.lineTo(cx+(cw*0.1)+fc,cy+fc);
      ctx.lineTo(cx+(cw*0.1)+fc,cy+(ch*0.333333));
      ctx.arc(cx+(cw/2),cy+ch-(cw*0.5),(cw*0.5)-fc,this.deg2rad(-80),this.deg2rad(180), false);
      break;
    case "6":
      ctx.moveTo(cx+fc,cy+ch-(cw*0.5)-fc);
      ctx.arc(cx+(cw/2),cy+ch-(cw*0.5),(cw*0.5)-fc,this.deg2rad(-180),this.deg2rad(180), false);
      ctx.bezierCurveTo(cx+fc,cy+fc,cx+fc,cy+fc,cx+(cw*0.9)-fc,cy+fc);
      ctx.moveTo(cx+(cw*0.9)-fc,cy+fc);
      break;
    case "7":
      ctx.moveTo(cx+(cw*0.5),cy+ch-fc);
      ctx.lineTo(cx+cw-fc,cy+fc);
      ctx.lineTo(cx+(cw*0.1)+fc,cy+fc);
      ctx.lineTo(cx+(cw*0.1)+fc,cy+(ch*0.25)-fc);
      break;
    case "8":
      ctx.moveTo(cx+(cw*0.92)-fc,cy+(cw*0.59));
      ctx.arc(cx+(cw/2),cy+(cw*0.45),(cw*0.45)-fc,this.deg2rad(25),this.deg2rad(-205), true);
      ctx.arc(cx+(cw/2),cy+ch-(cw*0.5),(cw*0.5)-fc,this.deg2rad(-135),this.deg2rad(-45), true);
      ctx.closePath();
      ctx.moveTo(cx+(cw*0.79),cy+(ch*0.47));
      ctx.lineTo(cx+(cw*0.21),cy+(ch*0.47));
      break;
    case "9":
      ctx.moveTo(cx+cw-fc,cy+(cw*0.5));
      ctx.arc(cx+(cw/2),cy+(cw*0.5),(cw*0.5)-fc,this.deg2rad(0),this.deg2rad(360), false);
      ctx.bezierCurveTo(cx+cw-fc,cy+ch-fc,cx+cw-fc,cy+ch-fc,cx+(cw*0.1)+fc,cy+ch-fc);
      break;
    case "%":
      ctx.moveTo(cx+fc,cy+(ch*0.75));
      ctx.lineTo(cx+cw-fc,cy+(ch*0.25));
      ctx.moveTo(cx+(cw*0.505),cy+(cw*0.3));
      ctx.arc(cx+(cw*0.3),cy+(cw*0.3),(cw*0.3)-fc,this.deg2rad(0),this.deg2rad(360), false);
      ctx.moveTo(cx+(cw*0.905),cy+ch-(cw*0.3));
      ctx.arc(cx+(cw*0.7),cy+ch-(cw*0.3),(cw*0.3)-fc,this.deg2rad(0),this.deg2rad(360), false);
      break;
    case ".":
      ctx.moveTo(cx+(cw*0.25),cy+ch-fc-fc);
      ctx.arc(cx+(cw*0.25),cy+ch-fc-fc,fc,this.deg2rad(0),this.deg2rad(360), false);
      ctx.closePath();
      break;
    default:
      break;
    }	
    ctx.stroke();
  },
  
  
  toHex:function(d, padding)
  {
    var hex = d.toString(16);
    
    padding = (typeof (padding) === "undefined" || padding === null ?
               padding = 2 : padding);
    while (hex.length < padding)
      hex = "0" + hex;
    return hex;
  },

  
  log:function(str)
  {
    /**/ if (typeof(       console)!='undefined')           console.log(str);
    else if (typeof(window.console)!='undefined')    window.console.log(str);
  }
};
